<?php
namespace JHodges\GiteaBotPHPUnit;

use \JHodges\GiteaBot\Client;

/* creates a new issue for each failed test
* if it already exists then report is added as a comment
* if it passes then the issue is closed
*/

final class PostFailedTests{

    private $fails=[];
    private $client=null;
    private $repo=null;
    private $title='';

    function __construct($xml_path){
        $this->xml = simplexml_load_file($xml_path);
    }

    public function post($url, $user, $pass, $repoUser, $repo, $title='Test Results'){
        if(!$url) return;

        $this->title=$title;

        // open connection and repo
        $this->client=new Client($url, $user, $pass, true);
        $this->repo=$this->client->getRepo($repoUser, $repo);

        $this->process($this->xml);
    }


    private function fail($name,$att,$error,$message){
        //composer the issue message
        $issue_title="$this->title $name ($error)";
        if(trim($message)) $message="```plain\n$message\n```\n";
        foreach($att as $k=>$v){
            $message.=" * $k = $v\n";
        }
        // upload any screenshots and add to message
        if(file_exists("/tmp/$name.png")){
            $data=$this->repo->addReleaseAttachment(1,"/tmp/$name.png");
            $url=$data->browser_download_url;
            $message="\n![]($url)\n".$message;
            unlink("/tmp/$name.png");
        }

        // does the issue exist
        $issue=$this->repo->getIssues(['q' => $issue_title])[0]??null;
        if($issue){
            $issue->addComment($message);
        }else{
            $issue=$this->repo->createIssue([
                'title'=>$issue_title,
                'body'=>$message
            ]);
        }
    }

    private function success($name,$att){
        $issue_title="$this->title $name";

        $issue=$this->repo->getIssues(['q' => $issue_title])[0]??null;

        if($issue){
            $issue->state='closed';
            $issue->save();
        }
    }

    private function process($item){
        if(isset($item->testcase)){
            foreach($item->testcase as $testcase) {
                $att=$testcase->attributes();
                $name=$testcase->attributes()->class[0].'::'.$testcase->attributes()->name[0];
                $fail=false;
                foreach($testcase as $k=>$v){
                    $this->fail($name,$att,$k,$v);
                    $fail=true;
                }
                if(!$fail){
                    $this->success($name,$att);
                }
            }
        }
        if(isset($item->testsuite)){
            foreach($item->testsuite as $testsuite) {
                $this->process($testsuite);
            }
        }
    }

}
